import pandas as pd
from eganalyze import settings


class EgData:

    def __init__(self, df):
        self.df = df
        self._normalize_column_names()
        self._parse_dates()
        self._enrich()
        self.df = self.df.sort_values(by='date_funded', ascending=False)

    def _parse_dates(self):
        datetime_columns = ['date_funded', 'next_payment_date', 'maturity_date', 'repaid_date']
        for col in datetime_columns:
            self.df[col] = pd.to_datetime(self.df[col])

    def _normalize_column_names(self):
        # "Interest Rate" becomes "interest_rate"
        self.df = self.df.rename(columns=lambda x: x.replace(' ', '_').lower())

    def _enrich(self):
        # Loan ID from Project Name
        self.df['id'] = [val[-1].strip() for val in self.df['project_name'].str.split(',')]

        # Build URL using Loan ID
        self.df['url'] = settings.LOAN_URL + self.df['id']

        # Calculate percentage of outstanding principal
        self.df['outstanding_principal_percentage'] = (self.df['outstanding_principal'] / self.df['outstanding_principal'].sum())

        # Calculate weighted interest rate
        self.df['outstanding_interest_rate_weighted'] = (self.df['outstanding_principal_percentage'] * self.df['interest_rate'])

        # Calculate weighted LTV
        self.df['outstanding_ltv_weighted'] = (self.df['outstanding_principal_percentage'] * self.df['ltv'])

    # LTV

    @property
    def mean_ltv(self):
        return float(self.df['ltv'].mean())

    @property
    def outstanding_mean_ltv(self):
        return float(self.df[self.df.outstanding_principal > 0]['ltv'].mean())
    
    @property
    def outstanding_weighted_mean_ltv(self):
        return float(self.df['outstanding_ltv_weighted'].sum())

    # Interest rate

    @property
    def mean_interest_rate(self):
        return float(self.df['interest_rate'].mean())

    @property
    def outstanding_mean_interest_rate(self):
        return float(self.df[self.df.outstanding_principal > 0]['interest_rate'].mean())

    @property
    def outstanding_weighted_mean_interest_rate(self):
        return float(self.df['outstanding_interest_rate_weighted'].sum())



import click
import pandas as pd
from eganalyze import __version__
from eganalyze.lib import EgData


@click.group()
@click.version_option(
    __version__, '--version', message="%(prog)s, version %(version)s"
)
def main():
    pass


@main.command()
@click.argument('input', type=click.Path(exists=True))
def analyze(input):

    data = EgData(pd.read_csv(input))

    click.echo(
        'Mean interest rate: {0:.2f}%'.format(data.mean_interest_rate)
    )
    click.echo(
        'Outstanding mean interest rate: {0:.2f}%'.format(data.outstanding_mean_interest_rate)
    )
    click.echo(
        'Outstanding weighted mean interest rate: {0:.2f}%'.format(data.outstanding_weighted_mean_interest_rate)
    )

    click.echo(
        'Mean LTV: {0:.2f}%'.format(data.mean_ltv)
    )
    click.echo(
        'Outstanding mean LTV: {0:.2f}%'.format(data.outstanding_mean_ltv)
    )
    click.echo(
        'Outstanding weighted mean LTV: {0:.2f}%'.format(data.outstanding_weighted_mean_ltv)
    )


from setuptools import setup
from eganalyze import __version__


setup(
    name='eganalyze',
    version=__version__,
    packages = ['eganalyze'],
    author='Kevin Kennell',
    author_email='kevin@kennell.de',
    license='MIT',
    url='https://github.com/kennell/eganalyze',
    install_requires=[
            'click',
            'pandas'
    ],
    entry_points={
        'console_scripts': [
            'eganalyze = eganalyze.cli:main'
        ]
    }
)
